################################################
## Objective function for the to be minimized ##
## Estimation of the noncentrality parameter  ##
################################################

fobj <- function(lambda,TestOss,df,gamma=0.5){

	(pchisq(q=TestOss,df=df,ncp=lambda)-gamma)^2

}

#########################
## Dependence Diagrams ##
#########################

DependenceDiagrams <- function(x,max.lag=50,num.clas="MW",alpha=0.05,alphaRP=0.5,step=5,max.height=TRUE,plot.ADF=T,plot.RP=T,plot.Cramer=T,plot.pstar=T){

# x: numeric vector related to the observed time series (or the series of residuals from a model)
# max.lag: the maximum lag considered
# num.clas: number of classes in each contingency table (as default, see equation (11) in the paper)
# alpha: the significance level of the tests of lag independence (related to each bar)
# 1-alphaRP: the coverage probability of the confidence interval for RP
# step: the step between x-ticks (default=5)
# max.height: max height in the autodependogram (if TRUE, an automatic rule is used)
# plot.ADF:    if TRUE, the autodependogram is plotted
# plot.RP:     if TRUE, the RP-autodependogram is plotted
# plot.Cramer: if TRUE, the Cramer-autodependogram is plotted
# plot.pstar:  if TRUE, the diagram of the transfromed p-values is plotted

library(Hmisc)

index <- length(x)

if(num.clas=="MW"){

	ks       <- floor(sqrt((index-max.lag)/5))
	kp       <- floor(sqrt(4*(2*(((index-1)^2)/(qnorm(alpha)^2)))^(1/5)))
	num.clas <- min(ks,kp)

}

if(num.clas>sqrt((index-max.lag)/5)){

	cat("warning: k is too large:\n")
	num.clas <- floor(sqrt((index-max.lag)/5))
	cat(paste("k=",num.clas,"\n",sep=""))

}

tab.freq      <- NULL
tab.freq.teo  <- NULL
test          <- NULL
comp.test     <- NULL
pvalue   <- numeric(max.lag)
ADF      <- numeric(max.lag)
lambda   <- numeric(max.lag)
Lower    <- numeric(max.lag)
Upper    <- numeric(max.lag)
RP       <- numeric(max.lag)
RPL      <- numeric(max.lag)
RPU      <- numeric(max.lag)
df       <- numeric(max.lag)
n        <- numeric(max.lag)
for(i in 1:max.lag){
	y             <- cut2(x[(i+1):index],g=num.clas)
  z             <- cut2(x[1:(index-i)],g=num.clas)
  tab.freq[[i]] <- table(y,z)
	n[i]          <- sum(tab.freq[[i]])
 
  cat(paste("Lag ",i,sep=""))
	cat("\n")
  cat("Frequenze Osservate")
	cat("\n")

	cont.table    <- cbind(tab.freq[[i]],Total=rowSums(as.matrix(tab.freq[[i]])))
	cont.table    <- rbind(cont.table,Total=colSums(cont.table))

	print(cont.table)
	cat("\n")
	
	tab.freq.teo[[i]] <- outer(rowSums(as.matrix(tab.freq[[i]])),colSums(as.matrix(tab.freq[[i]])),"*")/n[i]
	comp.test[[i]]    <- (tab.freq[[i]]-tab.freq.teo[[i]])^2/tab.freq.teo[[i]]

	print(comp.test[[i]])
	cat("\n")	

	test[[i]]     <- chisq.test(tab.freq[[i]])
  pvalue[i]     <- test[[i]]$p.value
	ADF[i]        <- test[[i]]$statistic
	df[i]         <- test[[i]]$parameter
	xmin          <- optimize(fobj, c(0, 1.5*ADF[i]+0.001), tol = 10^(-10),TestOss=ADF[i],df=df[i],gamma=0.5)         #1e+10
	xminL         <- optimize(fobj, c(0, 2*ADF[i]+0.001), tol = 10^(-10),TestOss=ADF[i],df=df[i],gamma=(1-alphaRP/2)) #1e+10
	xminU         <- optimize(fobj, c(0, 4*ADF[i]+0.001), tol = 10^(-10),TestOss=ADF[i],df=df[i],gamma=alphaRP/2)     #1e+10
	lambda[i]     <- xmin$minimum
	Lower[i]      <- xminL$minimum
	Upper[i]      <- xminU$minimum
	RP[i]         <- 1-pchisq(qchisq(1-alpha,df=df[1]),df=df[i],ncp=lambda[i])
	RPL[i]        <- 1-pchisq(qchisq(1-alpha,df=df[1]),df=df[i],ncp=Lower[i])
	RPU[i]        <- 1-pchisq(qchisq(1-alpha,df=df[1]),df=df[i],ncp=Upper[i])
  }
critvalue  <- rep(qchisq(1-alpha,df=df[1]),max.lag)
alpha      <- rep(alpha,max.lag)
num.clas   <- rep(num.clas,max.lag)

# p* as defined in equation (14) of the paper

pstar  <- (pvalue<alpha)*(-1/(2*alpha)*pvalue+1)+(pvalue>=alpha)*(1-pvalue)/(2*(1-alpha))
cramer <- sqrt(ADF/(n*(num.clas-1)))
critvalue.cramer <- sqrt(critvalue/(n*(num.clas-1)))

# Dummy indicators of rejection

reject <- (pvalue < alpha)*1 # if reject=1, then I have rejected

# Plot of the AutoDependogram

if(max.height==TRUE) lim <- max(1.01*critvalue,1.01*ADF)
else                 lim <- max.height

# --- #
# ADF #
# --- #

if(plot.ADF){
windows()
par(mai=c(0.84,0.8,0.012,0.004)) 
par(las = 3)
par(cex.axis=1.5) 
par(cex.lab=1.4)  
	plot(1:max.lag,ADF,main="",xlab="Lag",ylab="ADF",axes = FALSE,type = "h",ylim=c(0,lim),lwd=1)#,col=3-reject
	axis(1, at = seq(0,max.lag,by=step),label = seq(0,max.lag,by=step)) # at = 1:max.lag, label = 1:max.lag
	axis(2)
	box(col = "black")
	abline(a=0,b=0)
	abline(a=critvalue[1],b=0,col="blue",lty=2)
	}

# ------ #
# RP-ADF #
# ------ #

if(plot.RP){
windows()
par(mai=c(0.84,0.8,0.012,0.004)) 
par(las = 3)
par(cex.axis=1.5)
par(cex.lab=1.4)  
	plot(1:max.lag,RP,main="",xlab="Lag",ylab="RP",axes = FALSE,type = "h",ylim=c(0,1),lwd=1,col="white")
	axis(1, at = seq(0,max.lag,by=step),label = seq(0,max.lag,by=step)) # at = 1:max.lag, label = 1:max.lag
	axis(2)
	box(col = "black")
	abline(a=0,b=0)
  abline(a=1,b=0)
  rect(1:max.lag-0.21, rep(0,max.lag), 1:max.lag+0.21, RP, density = NULL, angle = 45, border = NULL, lty=1, lwd=2, col="gray")
	segments(1:max.lag, RPL, 1:max.lag, RPU)
	segments(1:max.lag-0.1, RPL, 1:max.lag+0.1, RPL)
	segments(1:max.lag-0.1, RPU, 1:max.lag+0.1, RPU)
	abline(a=1/2,b=0,col="blue",lty=2)
	abline(a=alpha,b=0,col="orange",lty=3)
  }

# ------ #
# Cramer #
# ------ #

if(plot.Cramer){
windows()
par(mai=c(0.84,0.8,0.012,0.004)) 
par(las = 3)
par(cex.axis=1.5) 
par(cex.lab=1.4)  
	plot(1:max.lag,cramer,main="",xlab="Lag",ylab="Cramer",axes = FALSE,type = "h",ylim=c(0,1),lwd=1)
	axis(1, at = seq(0,max.lag,by=step),label = seq(0,max.lag,by=step)) # at = 1:max.lag, label = 1:max.lag
	axis(2)
	box(col = "black")
	abline(a=0,b=0)
  abline(a=1,b=0)
  points(1:max.lag, critvalue.cramer,col="blue",pch=16,cex=0.7)
	}

# -- #
# p* #
# -- #

if(plot.pstar){
  windows()
  par(mai=c(0.84,0.8,0.012,0.004)) 
  par(las = 3)
  par(cex.axis=1.5) 
  par(cex.lab=1.4)  
  plot(1:max.lag,pstar,main="",xlab="Lag",ylab="Transformed p-values",axes = FALSE,type = "h",ylim=c(0,1),lwd=1)
  axis(1, at = seq(0,max.lag,by=step),label = seq(0,max.lag,by=step)) # at = 1:max.lag, label = 1:max.lag
  axis(2)
  box(col = "black")
  abline(a=0,b=0)
  abline(a=1,b=0)
  abline(a=1/2,b=0,col="blue",lty=2)
  }

result <- data.frame(
	lag             = 1:max.lag,        # considered lags 
	n               = n,                # effective size for each lag
	k               = num.clas,         # number of intervals for each marginal in the contingency tables
	df              = df,               # degrees of freedom in the chi-square asymptotic distribution 
	ADF             = ADF,              # autodependence function (chi-square statistic); see equation (2) in the paper
	critical.value  = critvalue,        # critical value of the chi-square test
	p.value         = pvalue,           # p-value of the chi-square test
	ncp             = lambda,           # delta "bullet" in the paper (see equation (6))   
	lower           = Lower,            # see equation (8) in the paper      
	upper           = Upper,            # see equation (9) in the paper 
	RP              = RP,               # RP-estimates 
	RP.lower        = RPL,              # see equation (10) in the paper
	RP.upper        = RPU,              # see equation (10) in the paper
	pstar           = pstar,            # transormed p-values; see equation (14) in the paper  
	Cramer          = cramer,           # normalized Cremer statistic (see equation (12) in the paper) 
	Cramer.crit.val = critvalue.cramer, # critical values, at each lag, for the Cremer statistics (see equation (12) in the paper)  
	alpha           = alpha,            # significance level
	rejection       = reject            # indicator variable of rejection; see equation (7) in the paper
)

return(result)

}
